from sensor.Sensor import Sensor
from utils.datatypes import *
from utils import i18n

import re
import os
import time
import Queue
import commands

class PrayerTimeSensor(Sensor):

   PRAYERSOURCE = "ipraytime --latitude %(latitude)s " \
                  "--longitude %(longitude)s --utcdiff %(timezone)s"
   REGSTART = ".*\]"
   REGMISC = "\s*\d+:\d+"
   REGEND = "\s*(?P<stime>\d+:\d+)\s*\S*"
   
   prayernames = [ "fajr", "zuhr", "asr", "maghrib", "isha" ]
   prayertimes = []
   nextcolor = "yellow"
   defcolor = "orange"
   nextprayer = -1
   icons = [ "icons/fajr.png", "icons/zuhr.png", "icons/asr.png",
             "icons/maghrib.png", "icons/isha.png" ]
   
   def __init__(self):
      global _; _ = i18n.Translator("praytime-sensor")
      self.__queue = Queue.Queue(1)
      self.__path = os.getcwd()

      Sensor.__init__(self)
      self._set_config_type("latitude", TYPE_STRING, "21.4206")
      self._set_config_type("longitude", TYPE_STRING, "39.8303")
      self._set_config_type("timezone", TYPE_STRING, "+2")
      self._set_config_type("location", TYPE_STRING, "Makkah, Saudi Arabia")

      self._add_thread(self.__get_prayertimes_thread)
      self._add_timer(0, self.__get_prayertimes)


   def get_configurator(self):
      configurator = self._new_configurator()
      configurator.set_name(_("Prayer Times"))
      configurator.add_title(_("Settings"))
      configurator.add_entry(_("Location:"), "location",
                             _("Name of the Location to be Displayed"))
      configurator.add_entry(_("Latitude:"), "latitude", _("Latitude"))
      configurator.add_entry(_("Longitude:"), "longitude", _("Longitude"))
      configurator.add_entry(_("Time Zone:"), "timezone", _("Hours from GMT"))
      return configurator

   def __adjust_night_clock(self, clock, num):
      tstr = time.strftime('%H:%M', clock)
      strinfo = tstr.split(":")
      hr = (int)(strinfo[0])
      typeoftime = " pm"
      if (hr > 12):
         hr = hr - 12
      if ((num == 1) and (hr == 11)):
         typeoftime = " am"
      if (num == 0):
         typeoftime = " am"
      tstr = (str)(hr) + ":" + strinfo[1]
      c = time.strptime(tstr, '%H:%M')
      return time.strftime('%H:%M', c) + typeoftime

   def __approachingNextSalat(self):
      str = __calculateDistance()
      tmp = str.split(":")
      hrdiff = (int)(tmp[0])
      mindiff = (int)(tmp[1])
      if (hrdiff < 1) and (mindiff < 1):
         return 1
      return 0

      
   def __calculateDistance(self):
      try:
         sclock = self.prayertimes
         active = self.nextprayer

         hrleft = (sclock[active])[3] - (time.localtime(time.time()))[3]
         minleft = (sclock[active])[4] - (time.localtime(time.time()))[4]
         if (hrleft < 0): hrleft = hrleft + 24
         if (minleft < 0):
            minleft = minleft + 60
            hrleft = hrleft - 1
         if (hrleft < 10): hrleft = '0' + (str)(hrleft)
         if (minleft < 10): minleft = '0' + (str)(minleft)
         return (str)(hrleft) + ":" + (str)(minleft)
      except:
         return "??:??"

      
   def __get_prayertimes(self):
      if (not self.__queue.empty()):
         output = self.__queue.get()
         self._send_output(output)
      self._add_timer(500, self.__get_prayertimes)

   def __get_prayertimes_thread(self):
      counter = 0
      old_values = ("", "", "", "")
      curtime = int(time.time())
      last_update = curtime - (15*60) - 1
      
      while (1):
         if (self._is_stopped()): break

         latitude = self._get_config("latitude")
         longitude = self._get_config("longitude")
         timezone = self._get_config("timezone")
         location = self._get_config("location")
         update_flag = 1

         curtime = int(time.time())
         if (((latitude, longitude, location, timezone) != old_values) or
             (last_update + 15 * 60 <= curtime) or update_flag):

            update_flag = 0
            old_values = (latitude, longitude, location, timezone)
            counter = 0
            last_update = int(time.time())

            output = self._new_output()            
            output.set("titlestr", _("Prayer Times"))
            output.set("location", location)
            output.set("stime", _("Time:"))
            output.set("nextstr", _("Next Prayer:"))

            output.set("fajrstr", _("Fajr:"))
            output.set("shuruqstr", _("Shuruq:"))
            output.set("zuhrstr", _("Dhuhr:"))
            output.set("asrstr", _("Asr:"))
            output.set("maghribstr", _("Maghrib:"))
            output.set("ishastr", _("Isha:"))

            for i in range(0, 5):
               output.set(self.prayernames[i]+"color", self.defcolor)

            fail, data = \
                  commands.getstatusoutput(self.PRAYERSOURCE % vars())

            try:
               stimes = [ '', '', '', '', '', '' ]
               
               for i in range(0, 6):
                  ctr = 0
                  regstr = "" + self.REGSTART
                  for j in range(0, i):
                     regstr = regstr + self.REGMISC
                  regstr = regstr + self.REGEND
                  regex = re.compile(regstr)
                  stimes[i] = regex.search(data).group("stime")

               output.set("shuruq", time.strftime('%H:%M',
                                time.strptime(stimes[1], '%H:%M')) + " am")

               tmp = stimes[0:1]
               tmp[1:] = stimes[2:]
               stimes = tmp
               sclock = [ '', '', '', '', '' ]

               for i in range(0, 5):
                  sclock[i] = time.strptime(stimes[i], '%H:%M')

               active = 0

               now = time.localtime(time.time())

               for i in range(0, 4):
                  if ((sclock[i][3:5] < now[3:5]) and
                      (sclock[i+1][3:5] > now[3:5])):
                     active = i+1
                     break

               output.set(self.prayernames[active]+"color", self.nextcolor)

               if (active == 0):
                  output.set("nextsalat", _("Fajr"))
               elif (active == 1):
                  output.set("nextsalat", _("Zuhr"))
               elif (active == 2):
                  output.set("nextsalat", _("Asr"))
               elif (active == 3):
                  output.set("nextsalat", _("Maghrib"))
               elif (active == 4):
                  output.set("nextsalat", _("Isha"))

               output.set("icon", os.path.join(self.__path,
                                               self.icons[active]))
               output.set(self.prayernames[0],
                          time.strftime('%H:%M', sclock[0]) + " am")
               for i in range(1, 5):
                  output.set(self.prayernames[i],
                                     self.__adjust_night_clock(sclock[i], i))


               output.set("nextsalatime",
                          self.__adjust_night_clock(sclock[active], active))

               self.prayertimes = sclock
               self.nextprayer = active

               strtmp = self.__calculateDistance()
               output.set("timeleft", strtmp)
               t = strtmp.split(":")
               output.set("timeleft_hm", t[0] + " hrs " + t[1] + " mins")

            except:
               output.set("shuruq", "??")
               for i in range(0, 5):
                  output.set(self.prayernames[i], "??")
               output.set("timeleft", "??:??")
               output.set("timeleft_hm", "?? hrs ?? mins")
               output.set("nextsalat", "??")
               output.set("nextsalatime", "??")
               output.set("icon", os.path.join(self.__path, self.icons[1]))
               
            self.__queue.put(output)

         elif ((counter % 60)==0):
            try:
               output = self._new_output()
               tl = self.__calculateDistance()
               output.set("timeleft", tl)
               t = tl.split(":")
               output.set("timeleft_hm", t[0] + " hrs " + t[1] + " mins")
               self.__queue.put(output)
            except:
               counter = counter + 1
         elif (self.__approachingNextSalat()):
            update_flag = 1

         counter = counter + 1
         time.sleep(1)


print """
PrayerTime Sensor 0.4
http://piousity.net
requires thamer mahmoud and ahmed twaijry's 
prayertime program and itl library,
obtainable from arabeyes.org.
"""

def new_sensor(args): return apply(PrayerTimeSensor, args)
